@@ -72,9 +72,9 @@
 #define E_CELL_COMBO_UTF8_KEY		"UTF-8-TEXT"
 
 
-static void e_cell_combo_class_init	(GtkObjectClass	*object_class);
+static void e_cell_combo_class_init	(GObjectClass	*object_class);
 static void e_cell_combo_init		(ECellCombo	*ecc);
-static void e_cell_combo_destroy	(GtkObject	*object);
+static void e_cell_combo_dispose	(GObject	*object);
 
 static gint e_cell_combo_do_popup	(ECellPopup	*ecp,
 					 GdkEvent	*event,
@@ -92,6 +92,10 @@ static void e_cell_combo_get_popup_pos	(ECellCombo	*ecc,
 					 gint		*height,
 					 gint		*width);
 
+static void e_cell_combo_selection_changed (GtkWidget *popup_list, ECellCombo *ecc);
+
+static gint e_cell_combo_list_button_press (GtkWidget *popup_list, GdkEvent *event, ECellCombo *ecc);
+
 static gint e_cell_combo_button_press	(GtkWidget	*popup_window,
 					 GdkEvent	*event,
 					 ECellCombo	*ecc);
@@ -111,19 +115,19 @@ static ECellPopupClass *parent_class;
 
 E_MAKE_TYPE (e_cell_combo, "ECellCombo", ECellCombo,
 	     e_cell_combo_class_init, e_cell_combo_init,
-	     e_cell_popup_get_type());
+	     e_cell_popup_get_type())
 
 
 static void
-e_cell_combo_class_init			(GtkObjectClass	*object_class)
+e_cell_combo_class_init			(GObjectClass	*object_class)
 {
 	ECellPopupClass *ecpc = (ECellPopupClass *) object_class;
 
-	object_class->destroy = e_cell_combo_destroy;
+	object_class->dispose = e_cell_combo_dispose;
 
 	ecpc->popup = e_cell_combo_do_popup;
 
-	parent_class = gtk_type_class (e_cell_popup_get_type ());
+	parent_class = g_type_class_ref (E_CELL_POPUP_TYPE);
 }
 
 
@@ -163,19 +167,27 @@ e_cell_combo_init			(ECellCombo	*ecc)
 					     gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (ecc->popup_scrolled_window)));
 	gtk_widget_show (ecc->popup_list);
 
-	gtk_signal_connect (GTK_OBJECT (ecc->popup_window),
-			    "button_press_event",
-			    GTK_SIGNAL_FUNC (e_cell_combo_button_press),
-			    ecc);
+	g_signal_connect (ecc->popup_list,
+			  "selection_changed",
+			  G_CALLBACK (e_cell_combo_selection_changed),
+			  ecc);
+	g_signal_connect (ecc->popup_list,
+			  "button_press_event",
+			  G_CALLBACK (e_cell_combo_list_button_press),
+			  ecc);
+	g_signal_connect (ecc->popup_window,
+			  "button_press_event",
+			  G_CALLBACK (e_cell_combo_button_press),
+			  ecc);
 	/* We use connect_after here so the list updates the selection before
 	   we hide the popup and update the cell. */
-	gtk_signal_connect_after (GTK_OBJECT (ecc->popup_window),
-				  "button_release_event",
-				  GTK_SIGNAL_FUNC (e_cell_combo_button_release),
-				  ecc);
-	gtk_signal_connect (GTK_OBJECT (ecc->popup_window),
-			    "key_press_event",
-			    GTK_SIGNAL_FUNC (e_cell_combo_key_press), ecc);
+	g_signal_connect (ecc->popup_window,
+			  "button_release_event",
+			  G_CALLBACK (e_cell_combo_button_release),
+			  ecc);
+	g_signal_connect (ecc->popup_window,
+			  "key_press_event",
+			  G_CALLBACK (e_cell_combo_key_press), ecc);
 }
 
 
@@ -189,17 +201,17 @@ e_cell_combo_init			(ECellCombo	*ecc)
 ECell *
 e_cell_combo_new			(void)
 {
-	ECellCombo *ecc = gtk_type_new (e_cell_combo_get_type ());
+	ECellCombo *ecc = g_object_new (E_CELL_COMBO_TYPE, NULL);
 
 	return (ECell*) ecc;
 }
 
 
 /*
- * GtkObject::destroy method
+ * GObject::dispose method
  */
 static void
-e_cell_combo_destroy			(GtkObject *object)
+e_cell_combo_dispose			(GObject *object)
 {
 	ECellCombo *ecc = E_CELL_COMBO (object);
 
@@ -207,7 +219,7 @@ e_cell_combo_destroy			(GtkObject *object)
 		gtk_widget_destroy (ecc->popup_window);
 	ecc->popup_window = NULL;
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -238,9 +250,9 @@ e_cell_combo_set_popdown_strings	(ECellCombo	*ecc,
 		gtk_widget_show (listitem);
 		gtk_container_add (GTK_CONTAINER (ecc->popup_list), listitem);
 
-		gtk_object_set_data_full (GTK_OBJECT (listitem),
-					  E_CELL_COMBO_UTF8_KEY,
-					  g_strdup (utf8_text), g_free);
+		g_object_set_data_full (G_OBJECT (listitem),
+					E_CELL_COMBO_UTF8_KEY,
+					g_strdup (utf8_text), g_free);
 
 		elem = elem->next;
 	}
@@ -257,8 +269,10 @@ e_cell_combo_do_popup			(ECellPopup	*ecp,
 	guint32 time;
 	gint error_code;
 
+	g_signal_handlers_block_by_func(ecc->popup_list, e_cell_combo_selection_changed, ecc);
 	e_cell_combo_show_popup (ecc, row, view_col);
 	e_cell_combo_select_matching_item (ecc);
+	g_signal_handlers_unblock_by_func(ecc->popup_list, e_cell_combo_selection_changed, ecc);
 
 	if (event->type == GDK_BUTTON_PRESS) {
 		GTK_LIST (ecc->popup_list)->drag_selection = TRUE;
@@ -298,7 +312,7 @@ e_cell_combo_select_matching_item	(ECellCombo	*ecc)
 
 	ecol = e_table_header_get_column (eti->header, ecp->popup_view_col);
 	cell_text = e_cell_text_get_text (ecell_text, ecv->e_table_model,
-					    ecol->col_idx, ecp->popup_row);
+					  ecol->col_idx, ecp->popup_row);
 
 	list = GTK_LIST (ecc->popup_list);
 	elem = list->children;
@@ -306,8 +320,8 @@ e_cell_combo_select_matching_item	(ECellCombo	*ecc)
 		listitem = GTK_WIDGET (elem->data);
 
 		/* We need to compare against the UTF-8 text. */
-		list_item_text = gtk_object_get_data (GTK_OBJECT (listitem),
-						      E_CELL_COMBO_UTF8_KEY);
+		list_item_text = g_object_get_data (G_OBJECT (listitem),
+						    E_CELL_COMBO_UTF8_KEY);
 
 		if (list_item_text && !strcmp (list_item_text, cell_text)) {
 			found = TRUE;
@@ -489,8 +503,33 @@ e_cell_combo_get_popup_pos		(ECellCombo	*ecc,
 		*height = work_height + list_requisition.height;
 }
 
+static void
+e_cell_combo_selection_changed(GtkWidget *popup_list, ECellCombo *ecc)
+{
+	if (!GTK_LIST(popup_list)->selection || !GTK_WIDGET_REALIZED(ecc->popup_window))
+		return;
 
+	e_cell_combo_restart_edit (ecc);
+}
 
+static gint
+e_cell_combo_list_button_press(GtkWidget *popup_list, GdkEvent *event, ECellCombo *ecc)
+{
+	g_return_val_if_fail (GTK_IS_LIST(popup_list), FALSE);
+
+	e_cell_combo_update_cell (ecc);
+	gtk_grab_remove (ecc->popup_window);
+	gdk_pointer_ungrab (event->button.time);
+	gtk_widget_hide (ecc->popup_window);
+
+	e_cell_popup_set_shown (E_CELL_POPUP (ecc), FALSE);
+	d(g_print("%s: popup_shown = FALSE\n", __FUNCTION__));
+
+	e_cell_combo_restart_edit (ecc);
+
+	return TRUE;
+
+}
 
 /* This handles button press events in the popup window.
    Note that since we have a pointer grab on this window, we also get button
@@ -622,8 +661,8 @@ e_cell_combo_update_cell		(ECellCombo	*ecc)
 
 	/* Get the text of the selected item. */
 	listitem = list->selection->data;
-	text = gtk_object_get_data (GTK_OBJECT (listitem),
-				    E_CELL_COMBO_UTF8_KEY);
+	text = g_object_get_data (G_OBJECT (listitem),
+				  E_CELL_COMBO_UTF8_KEY);
 	g_return_if_fail (text != NULL);
 
 	/* Compare it with the existing cell contents. */
