@@ -40,10 +40,10 @@ typedef void (*ETreeForeachFunc) (ETreePath path,
 typedef struct ETreeSelectionModelPriv ETreeSelectionModelPriv;
 
 #define E_TREE_SELECTION_MODEL_TYPE        (e_tree_selection_model_get_type ())
-#define E_TREE_SELECTION_MODEL(o)          (GTK_CHECK_CAST ((o), E_TREE_SELECTION_MODEL_TYPE, ETreeSelectionModel))
-#define E_TREE_SELECTION_MODEL_CLASS(k)    (GTK_CHECK_CLASS_CAST((k), E_TREE_SELECTION_MODEL_TYPE, ETreeSelectionModelClass))
-#define E_IS_TREE_SELECTION_MODEL(o)       (GTK_CHECK_TYPE ((o), E_TREE_SELECTION_MODEL_TYPE))
-#define E_IS_TREE_SELECTION_MODEL_CLASS(k) (GTK_CHECK_CLASS_TYPE ((k), E_TREE_SELECTION_MODEL_TYPE))
+#define E_TREE_SELECTION_MODEL(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), E_TREE_SELECTION_MODEL_TYPE, ETreeSelectionModel))
+#define E_TREE_SELECTION_MODEL_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST((k), E_TREE_SELECTION_MODEL_TYPE, ETreeSelectionModelClass))
+#define E_IS_TREE_SELECTION_MODEL(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), E_TREE_SELECTION_MODEL_TYPE))
+#define E_IS_TREE_SELECTION_MODEL_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), E_TREE_SELECTION_MODEL_TYPE))
 
 typedef struct {
 	ESelectionModel base;
@@ -56,7 +56,7 @@ typedef struct {
 } ETreeSelectionModelClass;
 
 
-GtkType          e_tree_selection_model_get_type            (void);
+GType            e_tree_selection_model_get_type            (void);
 ESelectionModel *e_tree_selection_model_new                 (void);
 void             e_tree_selection_model_foreach             (ETreeSelectionModel *etsm,
 							     ETreeForeachFunc     callback,
@@ -67,6 +67,7 @@ void             e_tree_selection_model_add_to_selection    (ETreeSelectionModel
 							     ETreePath            path);
 void             e_tree_selection_model_change_cursor       (ETreeSelectionModel *etsm,
 							     ETreePath            path);
+ETreePath        e_tree_selection_model_get_cursor          (ETreeSelectionModel *etsm);
 
 #ifdef __cplusplus
 }
